@@ -1,10 +1,11 @@
 package YAML::Syck;
 use strict;
-use vars qw( @ISA @EXPORT $VERSION $ImplicitTyping $SortKeys );
+use vars qw( @ISA @EXPORT $VERSION $ImplicitTyping $UseCode $LoadCode $DumpCode $SortKeys );
 use 5.00307;
+use Exporter;
 
 BEGIN {
-    $VERSION = '0.45';
+    $VERSION = '0.60';
     @EXPORT  = qw( Dump Load DumpFile LoadFile );
     @ISA     = qw( Exporter );
 
