@@ -70,12 +70,16 @@ yaml_syck_parser_handler
     SV *sv;
     AV *seq;
     HV *map;
-	struct parser_xtra *bonus = (struct parser_xtra *)p->bonus;
-	char load_code = bonus->load_code;
+    struct parser_xtra *bonus = (struct parser_xtra *)p->bonus;
+    char load_code = bonus->load_code;
     long i;
+    char *id = n->type_id;
+
+    while (id && (*id == '!')) { id++; }
+
     switch (n->kind) {
         case syck_str_kind:
-            if (TYPE_IS_NULL(n->type_id)) {
+            if (TYPE_IS_NULL(id)) {
                 if (strnEQ( n->data.str->ptr, NULL_LITERAL, 1+NULL_LITERAL_LENGTH)
                     && (n->data.str->style == scalar_plain)) {
                     sv = newSV(0);
@@ -83,19 +87,19 @@ yaml_syck_parser_handler
                     sv = newSVpvn(n->data.str->ptr, n->data.str->len);
                     CHECK_UTF8;
                 }
-            } else if (strEQ( n->type_id, "str" )) {
+            } else if (strEQ( id, "str" )) {
                 sv = newSVpvn(n->data.str->ptr, n->data.str->len);
                 CHECK_UTF8;
-            } else if (strEQ( n->type_id, "null" )) {
+            } else if (strEQ( id, "null" )) {
                 sv = newSV(0);
-            } else if (strEQ( n->type_id, "bool#yes" )) {
+            } else if (strEQ( id, "bool#yes" )) {
                 sv = newSVsv(&PL_sv_yes);
-            } else if (strEQ( n->type_id, "bool#no" )) {
+            } else if (strEQ( id, "bool#no" )) {
                 sv = newSVsv(&PL_sv_no);
-            } else if (strEQ( n->type_id, "default" )) {
+            } else if (strEQ( id, "default" )) {
                 sv = newSVpvn(n->data.str->ptr, n->data.str->len);
                 CHECK_UTF8;
-            } else if (strEQ( n->type_id, "float#base60" )) {
+            } else if (strEQ( id, "float#base60" )) {
                 char *ptr, *end;
                 UV sixty = 1;
                 NV total = 0.0;
@@ -119,21 +123,21 @@ yaml_syck_parser_handler
                 }
                 sv = newSVnv(total);
 #ifdef NV_NAN
-            } else if (strEQ( n->type_id, "float#nan" )) {
+            } else if (strEQ( id, "float#nan" )) {
                 sv = newSVnv(NV_NAN);
 #endif
 #ifdef NV_INF
-            } else if (strEQ( n->type_id, "float#inf" )) {
+            } else if (strEQ( id, "float#inf" )) {
                 sv = newSVnv(NV_INF);
-            } else if (strEQ( n->type_id, "float#neginf" )) {
+            } else if (strEQ( id, "float#neginf" )) {
                 sv = newSVnv(-NV_INF);
 #endif
-            } else if (strnEQ( n->type_id, "float", 5 )) {
+            } else if (strnEQ( id, "float", 5 )) {
                 NV f;
                 syck_str_blow_away_commas( n );
                 f = strtod( n->data.str->ptr, NULL );
                 sv = newSVnv( f );
-            } else if (strEQ( n->type_id, "int#base60" )) {
+            } else if (strEQ( id, "int#base60" )) {
                 char *ptr, *end;
                 UV sixty = 1;
                 UV total = 0;
@@ -156,17 +160,17 @@ yaml_syck_parser_handler
                     end = colon;
                 }
                 sv = newSVuv(total);
-            } else if (strEQ( n->type_id, "int#hex" )) {
+            } else if (strEQ( id, "int#hex" )) {
                 I32 flags = 0;
                 STRLEN len = n->data.str->len;
                 syck_str_blow_away_commas( n );
                 sv = newSVuv( grok_hex( n->data.str->ptr, &len, &flags, NULL) );
-            } else if (strEQ( n->type_id, "int#oct" )) {
+            } else if (strEQ( id, "int#oct" )) {
                 I32 flags = 0;
                 STRLEN len = n->data.str->len;
                 syck_str_blow_away_commas( n );
                 sv = newSVuv( grok_oct( n->data.str->ptr, &len, &flags, NULL) );
-            } else if (strnEQ( n->type_id, "int", 3 )) {
+            } else if (strnEQ( id, "int", 3 )) {
                 UV uv = 0;
                 syck_str_blow_away_commas( n );
                 if (grok_number( n->data.str->ptr, n->data.str->len, &uv) & IS_NUMBER_NEG) {
@@ -175,76 +179,109 @@ yaml_syck_parser_handler
                 else {
                     sv = newSVuv(uv);
                 }
+#ifdef PERL_LOADMOD_NOIMPORT
 #ifndef YAML_IS_JSON
-			} else if (load_code && strEQ(n->type_id, "perl/code:")) {
-				SV *cv;
-				SV *text, *sub;
+            } else if (load_code && (strEQ(id, "perl/code") || strnEQ(id, "perl/code:", 10))) {
+                SV *cv;
+                SV *text, *sub;
 
-				/* This code is copypasted from Storable.xs */
+                /* This code is copypasted from Storable.xs */
 
-				/*
-				 * prepend "sub " to the source
-				 */
+                /*
+                 * prepend "sub " to the source
+                 */
 
-				text = newSVpvn(n->data.str->ptr, n->data.str->len);
+                text = newSVpvn(n->data.str->ptr, n->data.str->len);
 
-				sub = newSVpvn("sub ", 4);
-				sv_catpv(sub, SvPV_nolen(text)); /* XXX no sv_catsv! */
-				SvREFCNT_dec(text);
+                sub = newSVpvn("sub ", 4);
+                sv_catpv(sub, SvPV_nolen(text)); /* XXX no sv_catsv! */
+                SvREFCNT_dec(text);
 
-				ENTER;
-				SAVETMPS;
+                ENTER;
+                SAVETMPS;
 
-				cv = eval_pv(SvPV_nolen(sub), TRUE);
+                cv = eval_pv(SvPV_nolen(sub), TRUE);
 
-				if (cv && SvROK(cv) && SvTYPE(SvRV(cv)) == SVt_PVCV) {
-					sv = cv;
-				} else {
-					croak("code %s did not evaluate to a subroutine reference\n", SvPV_nolen(sub));
-				}
+                sv_2mortal(sub);
 
-				SvREFCNT_inc(sv); /* XXX seems to be necessary */
-				SvREFCNT_dec(sub);
+                if (cv && SvROK(cv) && SvTYPE(SvRV(cv)) == SVt_PVCV) {
+                    sv = cv;
+                } else {
+                    croak("code %s did not evaluate to a subroutine reference\n", SvPV_nolen(sub));
+                }
 
-				FREETMPS;
-				LEAVE;
+                char *pkg = id + 10;
+                if ( *pkg != '\0' ) {
+                    sv_bless(sv, gv_stashpv(pkg, TRUE));
+                }
 
-				/* END Storable */
+                SvREFCNT_inc(sv); /* XXX seems to be necessary */
+
+                FREETMPS;
+                LEAVE;
+
+                /* END Storable */
 
             } else if (strnEQ( n->data.str->ptr, REF_LITERAL, 1+REF_LITERAL_LENGTH)) {
                 /* type tag in a scalar ref */
-                char *lang = strtok(n->type_id, "/:");
+                char *lang = strtok(id, "/:");
                 char *type = strtok(NULL, "");
 
                 if (lang == NULL || (strEQ(lang, "perl"))) {
-					sv = newSVpv(type, 0);
+                    sv = newSVpv(type, 0);
                 } else {
                     sv = newSVpv(form("%s::%s", lang, type), 0);
                 }
+            } else if ( strnEQ( id, "perl/scalar:", 12 ) ) {
+                char *pkg = id + 12;
+                sv = newSVpvn(n->data.str->ptr, n->data.str->len);
+                CHECK_UTF8;
+
+                sv = newRV_inc(sv);
+                if ( *pkg != '\0' ) {
+                    sv_bless(sv, gv_stashpv(pkg, TRUE));
+                }
+#endif
 #endif
             } else {
-                /* croak("unknown node type: %s", n->type_id); */
+                /* croak("unknown node type: %s", id); */
                 sv = newSVpvn(n->data.str->ptr, n->data.str->len);
                 CHECK_UTF8;
             }
         break;
 
         case syck_seq_kind:
+            /* load the seq into a new AV and place a ref to it in the SV */
             seq = newAV();
             for (i = 0; i < n->data.list->idx; i++) {
                 SV *a = perl_syck_lookup_sym(p, syck_seq_read(n, i));
                 av_push(seq, a);
                 USE_OBJECT(a);
             }
+            /* create the ref to the new array in the sv */
             sv = newRV_noinc((SV*)seq);
 #ifndef YAML_IS_JSON
-            if (n->type_id) {
-                char *lang = strtok(n->type_id, "/:");
+
+            if (id) {
+                /* bless it if necessary */
+                char *lang = strtok(id, "/:");
                 char *type = strtok(NULL, "");
-                while ((type != NULL) && *type == '@') { type++; }
+
+                if ( type != NULL ) {
+                    if (strnEQ(type, "array:", 6)) {
+                        /* !perl/array:Foo::Bar blesses into Foo::Bar */
+                        type += 6;
+                    }
+                    
+                    /* FIXME deprecated - here compatibility with @Foo::Bar style blessing */
+                    while ( *type == '@' ) { type++; }
+                }
 
                 if (lang == NULL || (strEQ(lang, "perl"))) {
-                    sv_bless(sv, gv_stashpv(type, TRUE));
+                    /* !perl/array on it's own causes no blessing */
+                    if ( !strEQ(type, "array") ) {
+                        sv_bless(sv, gv_stashpv(type, TRUE));
+                    }
                 } else {
                     sv_bless(sv, gv_stashpv(form("%s::%s", lang, type), TRUE));
                 }
@@ -254,20 +291,42 @@ yaml_syck_parser_handler
 
         case syck_map_kind:
 #ifndef YAML_IS_JSON
-            if ( (n->type_id != NULL) && (strEQ( n->type_id, "perl/ref:" ) ) ) {
+            if ( (id != NULL) && (strEQ(id, "perl/ref") || strnEQ( id, "perl/ref:", 9 ) ) ) {
+                /* handle scalar references, that are a weird type of mappings */
                 SV* key = perl_syck_lookup_sym(p, syck_map_read(n, map_key, 0));
                 SV* val = perl_syck_lookup_sym(p, syck_map_read(n, map_value, 0));
-                char *ref_type = SvPVX(key);
 
                 sv = newRV_noinc(val);
                 USE_OBJECT(val);
-                if (strnNE(ref_type, REF_LITERAL, REF_LITERAL_LENGTH+1)) {
+
+                char *ref_type = SvPVX(key);
+
+                if (strnNE(ref_type, REF_LITERAL, REF_LITERAL_LENGTH+1)) { /* handle the weird audrey scalar ref stuff */
                     sv_bless(sv, gv_stashpv(ref_type, TRUE));
+                } else {
+                    /* bless it if necessary */
+                    char *lang = strtok(id, "/:");
+                    char *type = strtok(NULL, "");
+
+                    if ( type != NULL && strnEQ(type, "ref:", 4)) {
+                        /* !perl/ref:Foo::Bar blesses into Foo::Bar */
+                        type += 4;
+                    }
+
+                    if (lang == NULL || (strEQ(lang, "perl"))) {
+                        /* !perl/ref on it's own causes no blessing */
+                        if ( !strEQ(type, "ref:") && (*type != NULL)) {
+                            sv_bless(sv, gv_stashpv(type, TRUE));
+                        }
+                    } else {
+                        sv_bless(sv, gv_stashpv(form("%s::%s", lang, type), TRUE));
+                    }
                 }
             }
             else
 #endif
             {
+                /* load the map into a new HV and place a ref to it in the SV */
                 map = newHV();
                 for (i = 0; i < n->data.pairs->idx; i++) {
                     SV* key = perl_syck_lookup_sym(p, syck_map_read(n, map_key, i));
@@ -278,16 +337,27 @@ yaml_syck_parser_handler
                 }
                 sv = newRV_noinc((SV*)map);
 #ifndef YAML_IS_JSON
-                if (n->type_id) {
-                    char *lang = strtok(n->type_id, "/:");
+                if (id)  {
+                    /* bless it if necessary */
+                    char *lang = strtok(id, "/:");
                     char *type = strtok(NULL, "");
-                    if (lang == NULL || strEQ(lang, "perl")) { /*  || (strchr(lang, '.') != NULL)) { */
-                        sv_bless(sv, gv_stashpv(type, TRUE));
+
+                    if ( type != NULL ) {
+                        if (strnEQ(type, "hash:", 5)) {
+                            /* !perl/hash:Foo::Bar blesses into Foo::Bar */
+                            type += 5;
+                        }
+
+                        /* FIXME deprecated - here compatibility with %Foo::Bar style blessing */
+                        while ( *type == '%' ) { type++; }
                     }
-                    else if (type == NULL) {
-                        sv_bless(sv, gv_stashpv(lang, TRUE));
-                    }
-                    else {
+
+                    if (lang == NULL || (strEQ(lang, "perl"))) {
+                        /* !perl/hash on it's own causes no blessing */
+                        if ( !strEQ(type, "hash") ) /* WTF TRAILING NEWLINE?!!! FIXME */ {
+                            sv_bless(sv, gv_stashpv(type, TRUE));
+                        }
+                    } else {
                         sv_bless(sv, gv_stashpv(form("%s::%s", lang, type), TRUE));
                     }
                 }
@@ -426,7 +496,7 @@ LoadYAML
 
     bonus.objects = (AV*)sv_2mortal((SV*)newAV());
     bonus.utf8 = SvTRUE(unicode);
-	bonus.load_code = SvTRUE(use_code) || SvTRUE(load_code);
+    bonus.load_code = SvTRUE(use_code) || SvTRUE(load_code);
     parser->bonus = &bonus;
 
     v = syck_parse(parser);
@@ -454,32 +524,45 @@ yaml_syck_emitter_handler
     SV*  sv = (SV*)data;
     struct emitter_xtra *bonus = (struct emitter_xtra *)e->bonus;
     char* tag = bonus->tag;
-	char dump_code = bonus->dump_code;
+    char dump_code = bonus->dump_code;
     char* ref = NULL;
     svtype ty = SvTYPE(sv);
 
-#define OBJECT_TAG     "tag:perl:"
+#define OBJECT_TAG     "tag:!perl:"
     
     if (SvMAGICAL(sv)) {
         mg_get(sv);
     }
 
 #ifndef YAML_IS_JSON
+
+    /* Handle blessing into the right class */
     if (sv_isobject(sv)) {
         ref = savepv(sv_reftype(SvRV(sv), TRUE));
         *tag = '\0';
         strcat(tag, OBJECT_TAG);
         switch (SvTYPE(SvRV(sv))) {
-            case SVt_PVAV: { strcat(tag, "@"); break; }
-            case SVt_RV:   { strcat(tag, "$"); break; }
-            case SVt_PVCV: { strcat(tag, "code"); break; }
-            case SVt_PVGV: { strcat(tag, "glob"); break; }
+            case SVt_PVAV: { strcat(tag, "array:");  break; }
+            case SVt_PVHV: { strcat(tag, "hash:");   break; }
+            case SVt_PVCV: { strcat(tag, "code:");   break; }
+            case SVt_PVGV: { strcat(tag, "glob:");   break; }
+            /* flatten scalar ref objects so that they dump as !perl/scalar:Foo::Bar foo */
+            case SVt_PVMG: {
+                               if ( !SvROK(SvRV(sv)) ) {
+                                   strcat(tag, "scalar:"); sv = SvRV(sv);
+                                   break;
+                               } else {
+                                   strcat(tag, "ref:");
+                                   break;
+                               }
+                           }
         }
         strcat(tag, ref);
     }
 #endif
 
     if (SvROK(sv)) {
+        /* emit a scalar ref ??? XXX FIXME */
 #ifdef YAML_IS_JSON
         PERL_SYCK_EMITTER_HANDLER(e, (st_data_t)SvRV(sv));
 #else
@@ -487,13 +570,15 @@ yaml_syck_emitter_handler
             case SVt_PVAV:
             case SVt_PVHV:
             case SVt_PVCV: {
+                /* Arrays, hashes and code values are inlined, and will be wrapped by a ref in the undumping */
                 e->indent = 0;
                 syck_emit_item(e, (st_data_t)SvRV(sv));
                 e->indent = PERL_SYCK_INDENT_LEVEL;
                 break;
             }
             default: {
-                syck_emit_map(e, "tag:perl:ref:", MAP_NONE);
+                syck_emit_map(e, OBJOF("tag:!perl:ref:"), MAP_NONE);
+                *tag = '\0';
                 syck_emit_item( e, (st_data_t)newSVpvn_share(REF_LITERAL, REF_LITERAL_LENGTH, 0) );
                 syck_emit_item( e, (st_data_t)SvRV(sv) );
                 syck_emit_end(e);
@@ -502,34 +587,40 @@ yaml_syck_emitter_handler
 #endif
     }
     else if (ty == SVt_NULL) {
+        /* emit an undef */
         syck_emit_scalar(e, "string", scalar_none, 0, 0, 0, NULL_LITERAL, NULL_LITERAL_LENGTH);
     }
     else if (SvNIOKp(sv) && (sv_len(sv) != 0)) {
+        /* emit a number with a stringified version */
         syck_emit_scalar(e, OBJOF("string"), SCALAR_NUMBER, 0, 0, 0, SvPV_nolen(sv), sv_len(sv));
     }
     else if (SvPOKp(sv)) {
+        /* emit a string */
         STRLEN len = sv_len(sv);
         if (len == 0) {
             syck_emit_scalar(e, OBJOF("string"), SCALAR_QUOTED, 0, 0, 0, "", 0);
         }
 #ifndef YAML_IS_JSON
-        else if ((len == NULL_LITERAL_LENGTH) && *(SvPV_nolen(sv)) == '~') {
+        else if (strEQ(SvPV_nolen(sv), NULL_LITERAL)) {
+            /* escape ~ as a quoted string */
             syck_emit_scalar(e, OBJOF("string"), SCALAR_QUOTED, 0, 0, 0, NULL_LITERAL, 1);
         }
 #endif
         else if (COND_FOLD(sv)) {
+            /* if we support UTF8 and the string contains UTF8 */
             enum scalar_style old_s = e->style;
             e->style = SCALAR_UTF8;
             syck_emit_scalar(e, OBJOF("string"), SCALAR_STRING, 0, 0, 0, SvPV_nolen(sv), len);
             e->style = old_s;
         }
         else {
+            /* just a simple string */
             syck_emit_scalar(e, OBJOF("string"), SCALAR_STRING, 0, 0, 0, SvPV_nolen(sv), len);
         }
     }
     else {
         switch (ty) {
-            case SVt_PVAV: {
+            case SVt_PVAV: { /* array */
                 syck_emit_seq(e, OBJOF("array"), SEQ_NONE);
                 e->indent = PERL_SYCK_INDENT_LEVEL;
 
@@ -547,7 +638,7 @@ yaml_syck_emitter_handler
                 syck_emit_end(e);
                 return;
             }
-            case SVt_PVHV: {
+            case SVt_PVHV: { /* hash */
                 HV *hv = (HV*)sv;
                 syck_emit_map(e, OBJOF("hash"), MAP_NONE);
                 e->indent = PERL_SYCK_INDENT_LEVEL;
@@ -569,7 +660,7 @@ yaml_syck_emitter_handler
                         HE *he = hv_iternext(hv);
 #endif
                         SV *key = hv_iterkeysv(he);
-                        av_store(av, AvFILLp(av)+1, key);	/* av_push(), really */
+                        av_store(av, AvFILLp(av)+1, key); /* av_push(), really */
                     }
                     STORE_HASH_SORT;
                     for (i = 0; i < len; i++) {
@@ -605,103 +696,106 @@ yaml_syck_emitter_handler
                 syck_emit_end(e);
                 return;
             }
-            case SVt_PVCV: {
-                /* XXX TODO XXX */
+            case SVt_PVCV: { /* code */
 #ifdef YAML_IS_JSON
                 syck_emit_scalar(e, "string", scalar_none, 0, 0, 0, NULL_LITERAL, NULL_LITERAL_LENGTH);
 #else
-	
-				/* This following code is mostly copypasted from Storable */
-				if ( !dump_code ) {
-					syck_emit_scalar(e, "tag:perl:code:", SCALAR_QUOTED, 0, 0, 0, "{ \"DUMMY\" }", 11);
-				} else {
-					dSP;
-					I32 len;
-					int count, reallen;
-					SV *text, *bdeparse;
-					CV *cv = (CV*)sv;
 
-					/*
-					 * Require B::Deparse. At least B::Deparse 0.61 is needed for
-					 * blessed code references.
-					 */
-					/* Ownership of both SVs is passed to load_module, which frees them. */
-					load_module(PERL_LOADMOD_NOIMPORT, newSVpvn("B::Deparse",10), newSVnv(0.61));
+                /* This following code is mostly copypasted from Storable */
+                if ( !dump_code ) {
+                    syck_emit_scalar(e, OBJOF("tag:!perl:code:"), SCALAR_QUOTED, 0, 0, 0, "{ \"DUMMY\" }", 11);
+                }
+#ifdef PERL_LOADMOD_NOIMPORT
+                else {
+                    dSP;
+                    I32 len;
+                    int count, reallen;
+                    SV *text, *bdeparse;
+                    CV *cv = (CV*)sv;
 
-					ENTER;
-					SAVETMPS;
+                    /*
+                     * Require B::Deparse. At least B::Deparse 0.61 is needed for
+                     * blessed code references.
+                     */
+                    /* Ownership of both SVs is passed to load_module, which frees them. */
+                    load_module(PERL_LOADMOD_NOIMPORT, newSVpvn("B::Deparse",10), newSVnv(0.61));
 
-					/*
-					 * create the B::Deparse object
-					 */
+                    ENTER;
+                    SAVETMPS;
 
-					PUSHMARK(sp);
-					XPUSHs(sv_2mortal(newSVpvn("B::Deparse",10)));
-					PUTBACK;
-					count = call_method("new", G_SCALAR);
-					SPAGAIN;
-					if (count != 1)
-						croak("Unexpected return value from B::Deparse::new\n");
-					bdeparse = POPs;
+                    /*
+                     * create the B::Deparse object
+                     */
 
-					/*
-					 * call the coderef2text method
-					 */
+                    PUSHMARK(sp);
+                    XPUSHs(sv_2mortal(newSVpvn("B::Deparse",10)));
+                    PUTBACK;
+                    count = call_method("new", G_SCALAR);
+                    SPAGAIN;
+                    if (count != 1)
+                        croak("Unexpected return value from B::Deparse::new\n");
+                    bdeparse = POPs;
 
-					PUSHMARK(sp);
-					XPUSHs(bdeparse); /* XXX is this already mortal? */
-					XPUSHs(sv_2mortal(newRV_inc((SV*)cv)));
-					PUTBACK;
-					count = call_method("coderef2text", G_SCALAR);
-					SPAGAIN;
-					if (count != 1)
-						croak("Unexpected return value from B::Deparse::coderef2text\n");
+                    /*
+                     * call the coderef2text method
+                     */
 
-					text = POPs;
-					len = SvLEN(text);
-					reallen = strlen(SvPV_nolen(text));
+                    PUSHMARK(sp);
+                    XPUSHs(bdeparse); /* XXX is this already mortal? */
+                    XPUSHs(sv_2mortal(newRV_inc((SV*)cv)));
+                    PUTBACK;
+                    count = call_method("coderef2text", G_SCALAR);
+                    SPAGAIN;
+                    if (count != 1)
+                        croak("Unexpected return value from B::Deparse::coderef2text\n");
 
-					/*
-					 * Empty code references or XS functions are deparsed as
-					 * "(prototype) ;" or ";".
-					 */
+                    text = POPs;
+                    len = SvLEN(text);
+                    reallen = strlen(SvPV_nolen(text));
 
-					if (len == 0 || *(SvPV_nolen(text)+reallen-1) == ';') {
-						croak("The result of B::Deparse::coderef2text was empty - maybe you're trying to serialize an XS function?\n");
-					}
+                    /*
+                     * Empty code references or XS functions are deparsed as
+                     * "(prototype) ;" or ";".
+                     */
 
-					/* 
-					 * Signal code by emitting SX_CODE.
-					 */
+                    if (len == 0 || *(SvPV_nolen(text)+reallen-1) == ';') {
+                        croak("The result of B::Deparse::coderef2text was empty - maybe you're trying to serialize an XS function?\n");
+                    }
+
+                    /* 
+                     * Signal code by emitting SX_CODE.
+                     */
 
 #if 0
-					/* SYCK adds anchors for us automatically */
+                    /* SYCK adds anchors for us automatically */
 
-					PUTMARK(SX_CODE);
-					cxt->tagnum++;   /* necessary, as SX_CODE is a SEEN() candidate */
+                    PUTMARK(SX_CODE);
+                    cxt->tagnum++;   /* necessary, as SX_CODE is a SEEN() candidate */
 #endif
 
-					/*
-					 * Now store the source code.
-					 */
+                    /*
+                     * Now store the source code.
+                     */
 
-					syck_emit_scalar(e, "tag:perl:code:", SCALAR_UTF8, 0, 0, 0, SvPV_nolen(text), len-1);
+                    syck_emit_scalar(e, OBJOF("tag:!perl:code:"), SCALAR_UTF8, 0, 0, 0, SvPV_nolen(text), len-1);
 
-					FREETMPS;
-					LEAVE;
+                    FREETMPS;
+                    LEAVE;
 
-					/* END Storable */
-				}
+                    /* END Storable */
+                }
 #endif
+#endif
+                *tag = '\0';
                 break;
             }
-            case SVt_PVGV:
-            case SVt_PVFM: {
+            case SVt_PVGV:   /* glob (not a filehandle, a symbol table entry) */
+            case SVt_PVFM: { /* format */
                 /* XXX TODO XXX */
                 syck_emit_scalar(e, OBJOF("string"), SCALAR_STRING, 0, 0, 0, SvPV_nolen(sv), sv_len(sv));
                 break;
             }
-            case SVt_PVIO: {
+            case SVt_PVIO: { /* filehandle */
                 syck_emit_scalar(e, OBJOF("string"), SCALAR_STRING, 0, 0, 0, SvPV_nolen(sv), sv_len(sv));
                 break;
             }
@@ -744,7 +838,8 @@ DumpYAML
 
     bonus.port = out;
     New(801, bonus.tag, 512, char);
-	bonus.dump_code = SvTRUE(use_code) || SvTRUE(dump_code);
+    *(bonus.tag) = '\0';
+    bonus.dump_code = SvTRUE(use_code) || SvTRUE(dump_code);
     emitter->bonus = &bonus;
 
     syck_emitter_handler( emitter, PERL_SYCK_EMITTER_HANDLER );
@@ -766,9 +861,11 @@ DumpYAML
     }
 #endif
 
+#ifdef SvUTF8_on
     if (SvTRUE(unicode)) {
         SvUTF8_on(out);
     }
+#endif
 
     FREETMPS; LEAVE;
 
